import atexit
import functools
import time
import uuid
from datetime import datetime
from typing import Callable

from .calltrack import ConsoleCalltrack, JSONCalltrack
from .dependencies import json_calltrack

atexit.register(lambda: json_calltrack.flush() if json_calltrack.queue.queue else None)


def consolelog(func: Callable):
    @functools.wraps(func)
    def consolelog_wrapper(*args, **kwargs):
        calltrack = ConsoleCalltrack()

        args_repr = [repr(a) for a in args]
        kwargs_repr = [f"{k}={repr(v)}" for k, v in kwargs.items()]
        signature = ", ".join(args_repr + kwargs_repr)

        timestamp = datetime.now()
        start_time = time.perf_counter()

        output = func(*args, **kwargs)

        end_time = time.perf_counter()
        run_time = end_time - start_time

        calltrack.append(f"Calling {func.__name__}({signature})")
        calltrack.append(f"{func.__name__}() returned {repr(output)}")
        calltrack.append(f"Computation time: {run_time}")
        calltrack.append(f"Timestamp: {timestamp}")

        calltrack.flush()

        return output

    return consolelog_wrapper


def jsonlog(
    _func=None,
    *,
    calltrack: JSONCalltrack | None = None,
):
    if not calltrack:
        calltrack = json_calltrack

    def jsonlog_decorator(func: Callable, calltrack: JSONCalltrack):
        @functools.wraps(func)
        def jsonlog_wrapper(*args, **kwargs):
            args_repr = [repr(a) for a in args]
            kwargs_repr = [f"{k}={repr(v)}" for k, v in kwargs.items()]
            signature = ", ".join(args_repr + kwargs_repr)

            uuidv1 = uuid.uuid1()

            timestamp = datetime.now()
            start_time = time.perf_counter()

            output = func(*args, **kwargs)

            end_time = time.perf_counter()
            run_time = end_time - start_time

            calltrack.append(
                log={
                    str(uuidv1): {
                        "name": func.__name__,
                        "args": args_repr,
                        "kwargs": kwargs_repr,
                        "signature": signature,
                        "output": repr(output),
                        "run_time": run_time,
                        "timestamp": str(timestamp),
                    }
                }
            )
            if calltrack.autoflush:
                calltrack.write(calltrack.log_queue.popleft())

            return output

        return jsonlog_wrapper

    if _func is None:
        return lambda func: jsonlog_decorator(func, calltrack=calltrack)
    else:
        return jsonlog_decorator(_func, calltrack=calltrack)



import json
import os
import warnings
from abc import ABC, abstractmethod
from datetime import datetime
from typing import Any

from .log_queue import LogQueue
from .utils import is_path_creatable


class Calltrack(ABC):
    """Calltrack Abstract Class"""

    def __init__(self, queue: LogQueue = LogQueue()):
        self.queue = queue

    def flush(self) -> None:
        while self.queue.queue:
            self.write(self.queue.popleft())

    def append(self, log: dict) -> None:
        self.queue.append(log)

    @abstractmethod
    def write(
        self,
        log: Any,
    ) -> None:
        """Write logs"""


class ConsoleCalltrack(Calltrack):
    def write(self, log: Any) -> None:
        print(log)


class JSONCalltrack(Calltrack):
    def __init__(
        self, save_dir: str, queue: LogQueue = LogQueue(), autoflush: bool = False
    ) -> None:
        super().__init__(queue=queue)
        self._save_dir = save_dir
        self.autoflush = autoflush

        self._prepare_dir()

    @property
    def save_dir(self):
        return self._save_dir

    @save_dir.setter
    def save_dir(self, path: str) -> None:
        if is_path_creatable(path):
            self._save_dir = path
            self._prepare_dir()
        else:
            warnings.warn("The provided path is ill-formed.")

    def flush(self, filename: str | None = None):
        concat_log = {}
        while self.queue.queue:
            log = self.queue.queue.popleft()
            self._check_log(log)
            concat_log.update(log)
        self.write(concat_log, filename)

    def write(self, log: dict, filename: str | None = None) -> None:
        self._check_log(log)
        """Assuming the log is well formatted"""
        self._export_log(log, filename)

    def _export_log(self, log: dict, filename: str | None = None) -> None:
        """Export the logs to a JSON file.

        Args:
        ----
            file_path (str, optional): The file path to save the logs. If not provided, a default file path will be used.

        """
        if not filename:
            filename = self._default_filename()
        if not filename.endswith(".json"):
            filename = filename + ".json"

        filepath = os.path.join(self.save_dir, filename)

        with open(filepath, "w") as file:
            json.dump(log, file, indent=4)

    def _default_filename(self) -> str:
        timestamp = datetime.now().strftime("%Y%m%d_%H%M%S_%f")
        return f"log_{timestamp}.json"

    def _prepare_dir(self) -> None:
        os.makedirs(os.path.join(os.getcwd(), self.save_dir), exist_ok=True)

    def _check_log(self, log: dict) -> None:
        if not isinstance(log, dict):
            raise TypeError(
                f"The log should be a dict. It is received as a {type(log)}. Value= {log}"
            )



import uuid
from collections import deque
from typing import Any, Deque


class LogQueue:
    def __init__(self, name: str | None = None):
        self.name = name if name else "default"
        self.id = uuid.uuid4()
        self.queue: Deque = deque([])

    def append(self, log: Any) -> None:
        """Add the log on top of a stack."""
        self.queue.append(log)

    def popleft(self):
        return self.queue.popleft()

    def __len__(self):
        return len(self.queue)



import os


def is_path_creatable(pathname: str) -> bool:
    """
    `True` if the current user has sufficient permissions to create the passed
    pathname; `False` otherwise.
    """
    # Parent directory of the passed path. If empty, we substitute the current
    # working directory (CWD) instead.
    dirname = os.path.dirname(pathname) or os.getcwd()
    return os.access(dirname, os.W_OK)



import json
import os
from datetime import datetime
from tkinter import Tk, filedialog

import streamlit as st


def read_json_file(file):
    with open(file) as f:
        return json.load(f)


def select_directory():
    root = Tk()
    root.withdraw()  #
    directory = filedialog.askdirectory()
    root.destroy()
    return directory


def view():
    st.set_page_config(layout="wide")
    st.title("Calltrack View")

    process_directory()


def process_directory():
    st.sidebar.header("Select Logs Directory")
    logs_path_input = st.sidebar.text_input("Enter logs directory path:", "")
    logs_path = logs_path_input

    if logs_path:
        json_files = [f for f in os.listdir(logs_path) if f.endswith(".json")]
        json_files.sort(
            key=lambda x: datetime.strptime(
                x.split("_", 1)[1].split(".")[0], "%Y%m%d_%H%M%S_%f"
            ),
            reverse=True,
        )

        selected_files = st.sidebar.multiselect("Select JSON files", json_files)

        if len(selected_files) > 0:
            cols = st.columns(len(selected_files))
            for i, file in enumerate(selected_files):
                file_path = os.path.join(logs_path, file)
                data = read_json_file(file_path)
                with cols[i]:
                    st.subheader(f"File: {file}")
                    for idx, obj in enumerate(data):
                        with st.expander(f"### Step {idx+1}"):
                            for key, value in data[obj].items():
                                st.markdown(f"**{key}:** {value}".replace("\n", ""))
        else:
            st.write("Please select files to compare.")
    else:
        st.write("Please provide a valid logs directory path.")


if __name__ == "__main__":
    view()



import os
import subprocess


def main():
    script_dir = os.path.dirname(os.path.realpath(__file__))
    target_file_path = os.path.join(script_dir, "view.py")

    command = ["streamlit", "run", target_file_path]
    subprocess.run(command)


if __name__ == "__main__":
    main()
